%   Inputs
%       dt:   timestep size (10 femtoseconds recommended)
%       N:    number of atoms in the simulation
%       Tdes: desired system temperature (100K recommended)
%   
%   Constants:
%       amass: atomic mass (6.634e-26 kg/atom for argon)
%       sigma: atomic diameter (.3405e-9m for argon)
%       kb: Boltzmann's constant (1.380e-23 J/K*molecule)
%       epsilon: LJ potential (kb*119.8K)
%           so epsilon's units are J/molecule
%
dt = 1.e-14;
dtq2 = dt/2;
dtsq = dt^2;
dtsq2 = dtsq/2;
amass = 6.634e-26;
amassinv = 1./amass;
sigma = 0.3405e-9;
sigmainv = 1./sigma;
sigsq = sigma^2;
kb = 1.380e-23;
epsilon = 119.8*kb;
eps24 = 24.*epsilon;
%
% Set initial positions as a function of sigma
%
%   Using the MATLAB 'ndgrid' routine and subsequent modifier 
%
N = input('Enter the number of atoms you wish to model:  ');
if N<8; N=8; end;
N3rd = round(N^(1/3));
disp('Number of atoms adjusted to the following for a proper initial grid:')
N = N3rd^3;
disp(N)
NL = 1.25*(N3rd-1)/2;
bigeyes = 1.e10*eye(N);
[rx,ry,rz] = ndgrid(-NL:1.25:NL, -NL:1.25:NL, -NL:1.25:NL);  % creates matrixed cartesian coordinates
boxLqs = 4.*NL;  %the periodic boundary can be adjusted if desired
axislim = boxLqs/2.;
rx = rx(:); ry = ry(:); rz = rz(:);  % modifier needed to change from matrix to vector mode
%
% Checking initial positions
%
% asize = 100.0; % in points... therefore not actual dimensions
figure(1);clf;
axis([-axislim axislim -axislim axislim -axislim axislim])
axis vis3d;
box on;
camlight;
view(30,15);
% axis manual;
Ar = scale(UnitSphere(2),.5,.5,.5);
Ar.facecolor = 'red'; Ar .facealpha = 1.0;
% Ar.facecolor = 'blue'; Ar .facealpha = 0.25;
% nuc = scale(UnitSphere(2),.05,.05,.05);
% nuc.facecolor = 'blue'; nuc.facealpha = 1.0;
% Ar = combine(Ar,nuc);
for i=1:N; 
    Ars = translate(Ar,rx(i),ry(i),rz(i));
    renderpatch(Ars);
end
% scatter3(rx,ry,rz,asize,'r','filled')
disp('Atom grid initialized... see plot then press any key to continue') 
pause
rx = sigma.*rx;
ry = sigma.*ry;
rz = sigma.*rz;
boxL = sigma*boxLqs;
qboxL = 1./boxL;
%
%  Computing a random initial velocity based on desired system temperature
%
Tdes = input('Enter the desired system temperature (100K recommended):  ');
Vxmean = sqrt(kb*Tdes/amass);
Vymean = Vxmean;
Vzmean = Vxmean;
%
% Set initial velocity distributions as a function of desired temperature, 
% Tdes (K)
%
Vx = Vxmean.*randn(N,1);
Vy = Vymean.*randn(N,1);
Vz = Vzmean.*randn(N,1);
%
% Correcting for velocity drift
%
Ninv = 1./N;
Vx = Vx - (Ninv*sum(Vx)).*ones(N,1);
Vy = Vy - (Ninv*sum(Vy)).*ones(N,1);
Vz = Vz - (Ninv*sum(Vz)).*ones(N,1);
disp('Actual initial system temperature from normal distribution comps:')
Tinit = (amass/(3*N*kb))*sum((Vx.*Vx)+(Vy.*Vy)+(Vz.*Vz));
disp(Tinit)
%
Vsq = 0.;
Timelapse = 0.;
kanim = 0;
viewstep = 30;
kdtend = input('Enter the desired number of time steps:  ');
for kdt = 1:kdtend
%    kanim = kanim + 1;
%    if kanim == 2
         drawnow
        rxplot = sigmainv*rx;
        ryplot = sigmainv*ry;
        rzplot = sigmainv*rz; 
%         scatter3(rxplot,ryplot,rzplot,asize,'b','filled')
%         axis manual;
        figure(1);clf;
        axis([-axislim axislim -axislim axislim -axislim axislim])
        axis vis3d;
        box on;
        camlight;
        view(viewstep,15);
        viewstep = viewstep + 1;
        if viewstep > 360; viewstep = 1; end;
        for i=1:N; 
            Ars = translate(Ar,rxplot(i),ryplot(i),rzplot(i));
            renderpatch(Ars);
        end       
%        kanim = 0;
%    end
    rxi = repmat(rx,1,N);
    ryi = repmat(ry,1,N);
    rzi = repmat(rz,1,N); 
    rxj = rxi';
    ryj = ryi';
    rzj = rzi';
    rxij = rxi - rxj;
    ryij = ryi - ryj;
    rzij = rzi - rzj;
    % correcting of distances for periodic boundaries
    rxij = rxij - boxL*round(qboxL*rxij);
    ryij = ryij - boxL*round(qboxL*ryij);
    rzij = rzij - boxL*round(qboxL*rzij);
    %
    % Force Computations
    %
    rijsq = rxij.*rxij + ryij.*ryij + rzij.*rzij;
    %
    % Diagonal Correction: 
    % Since i-i forces would compute as infinite, we add a large diagonal 
    % matrix to rijsq to simulate very large distances and therefore zero forces
    %
    rijsq = rijsq + bigeyes;
    rsqinv = 1./rijsq;
    sqr2 = sigsq.*rsqinv;
    sqr6 = sqr2.*sqr2.*sqr2;
    sqr12 = sqr6.*sqr6;
    fqrij = eps24.*rsqinv.*(2.*sqr12 - sqr6);
    fxij = fqrij.*rxij;
    fyij = fqrij.*ryij;
    fzij = fqrij.*rzij;
    fx = sum(fxij,2);
    fy = sum(fyij,2);
    fz = sum(fzij,2);
    ax = amassinv*fx;
    ay = amassinv*fy;
    az = amassinv*fz;
    % 
    %  Modified Verlet Algorithm for computing molecular displacements
    %
    %  ax, ay, az already computed based on rij
    %
    Vx = Vx + dtq2*ax;
    Vy = Vy + dtq2*ay;
    Vz = Vz + dtq2*az;
    rx = rx + dt*Vx + dtsq2*ax;
    ry = ry + dt*Vy + dtsq2*ay;
    rz = rz + dt*Vz + dtsq2*az;
    Vx = Vx + dtq2*ax;
    Vy = Vy + dtq2*ay;
    Vz = Vz + dtq2*az;
    %
    % Correcting of periodic boundary positions after movement is computed
    % 
    %   Periodic boundary logic:
    %       if(rx(i)> boxL/2) rx(i) = rx(i) - boxL; end
    %       if(rx(i)<-boxL/2) rx(i) = rx(i) + boxL; end
    %          and repeated for y and z dimensions
    %
    %   Instead do the following to cut the computation time down:
    %
    rx = rx - boxL*round(qboxL*rx);
    ry = ry - boxL*round(qboxL*ry);
    rz = rz - boxL*round(qboxL*rz);
    %
    %  Gathering average kinetic energies for system temperature checks
    %
    % if kdt >= 76
        Vsq = Vsq + sum((Vx.*Vx) + (Vy.*Vy) + (Vz.*Vz));
    % end
end  % End of the do loop for time iterations    
%
%  Updating scatter plot of atoms
%
% rxplot = sigmainv*rx;
% ryplot = sigmainv*ry;
% rzplot = sigmainv*rz; 
% scatter3(rxplot,ryplot,rzplot,asize,'b','filled')
% axis manual;
% axis([-axislim axislim -axislim axislim -axislim axislim])
% axis vis3d;
% box on;
disp('Time loop completed.  Elapsed time (in nanoseconds) is: ')
Timelapse = Timelapse + kdtend*dt*1e9;
disp(Timelapse)
Temp = (amass/(3*N*kb))*(Vsq/kdtend);
disp('Average system temperature over the time loop:')
disp(Temp)
disp('See updated plot then press any key to continue')
pause
disp('Run MDcont if you wish to continue this simulation')


