% value of GM for Earth
GM = 3.986004418e14;

% semi-major axis
a = 1.e7;

% eccentricity
e = 0.1;

% angle of apsides
omega_ap = pi/6;

% inclination
inc = pi/6;

% angle of nodes
Omega_node = pi/3;

% form rotation matrix
Rmat = rotmat(omega_ap,3) * rotmat(inc,1) * rotmat(Omega_node,3);


% Period, from Kepler's 3rd Law
P = sqrt(4 * pi^2 * a^3 / GM)
% time-step of orbit
N = 100;
dt = P/N;

% initial state-vector
xv0t = zeros(6,1);
% place planet on x-axis
% use form of ellipse
xv0t(1,1) = a*(1-e);
% velocity is in plus-y direction
% use energy conservation
xv0t(5,1) = sqrt(GM*(2. / xv0t(1,1) - 1. / a));

% rotated state-vector
xv0 = rotstate(Rmat,xv0t);

% set the initial plot
hold off;
close all;
scrsz = get(0,'ScreenSize');
figure1 = figure('Position',[0,scrsz(4)-512,512,512])

% plot an ellipse of the correct shape
ph = 0:pi/100.:2*pi;
r = a*(1-e^2) ./ (1+e*cos(ph));
xell = r .* cos(ph);
yell = r .* sin(ph);
xp = Rmat*[xell;yell;ph*0.];

plot3(xp(1,:),xp(2,:),xp(3,:),'-r');
axis equal;

% label the axes
xlabel('x position (meters)');
ylabel('y position (meters)');
zlabel('z position (meters)');
title('Earth Centered Orbit');

% set for overplotting
hold on;

% plot an "Earth" (not to scale)
R_E = 6.738e6;
plot3(0,0,0,'ob');

% Set the tolerances of the orbit integration
options = odeset('RelTol',5.e-8,'AbsTol',a*5.e-9);

% integrate the orbit
t = zeros(1,N);
xv = zeros(6,N+1);
t(1,1) = 0.;
xv(:,1) = xv0;
plot3(xv0(1,1),xv0(2,1),xv0(3,1),'.b');
Ar = scale(UnitSphere(4),R_E,R_E,R_E);
Ar.facecolor = 'blue';
camlight;
renderpatch(Ar);
for i = 1:N
   [ti,xvi] = ode45(@orbitfunc_complete,[0,dt],xv0,options);   
   nout = size(ti);
   t(1,i+1) = i*dt;
   xv(:,i+1) = xvi(nout(1),:)';
   plot3(xv(1,i+1),xv(2,i+1),xv(3,i+1),'.g');
   xv0(:,1) = xv(:,i+1);
   pause(0.05);
end

% verify that energy is conserved
figure2 = figure;
Energy = 0.5*sum(xv(4:6,:).^2,1) - GM ./ sqrt(sum(xv(1:3,:).^2,1));

E0 = -GM/(2*a);
plot(t,Energy/E0)
axis([0,P,0,1.1]);
xlabel('time');
ylabel('Energy/Energy_0');
title('Earth Centered Orbit');